@@ -1,4 +1,5 @@
 import itertools
+import pprint
 
 class _missing(object):
     pass
@@ -66,6 +67,11 @@ class Invalid(Exception):
             errors['.'.join(keyparts)] = '; '.join(msgs)
         return errors
 
+    def __str__(self):
+        """ Return a pretty-formatted string representation of the
+        result of an execution of this exception's ``asdict`` method"""
+        return pprint.pformat(self.asdict())
+
 class All(object):
     """ Composite validator which succeeds if none of its
     subvalidators raises an :class:`colander.Invalid` exception"""
@@ -98,13 +104,35 @@ class Range(object):
             if value < self.min:
                 raise Invalid(
                     node,
-                    '%r is less than minimum value %r' % (value, self.min))
+                    '%s is less than minimum value %s' % (value, self.min))
 
         if self.max is not None:
             if value > self.max:
                 raise Invalid(
                     node,
-                    '%r is greater than maximum value %r' % (value, self.max))
+                    '%s is greater than maximum value %s' % (value, self.max))
+
+class Length(object):
+    """ Validator which succeeds if the value passed to it has a
+    length between a minimum and maximum.  The value is most often a
+    string."""
+    def __init__(self, min=None, max=None):
+        self.min = min
+        self.max = max
+
+    def __call__(self, node, value):
+        if self.min is not None:
+            if len(value) < self.min:
+                raise Invalid(
+                    node,
+                    'Shorter than minimum length %s' % self.min)
+
+        if self.max is not None:
+            if len(value) > self.max:
+                raise Invalid(
+                    node,
+                    'Longer than maximum length %s' % self.max)
+
 
 class OneOf(object):
     """ Validator which succeeds if the value passed to it is one of
@@ -159,7 +187,7 @@ class Mapping(object):
         error = None
         result = {}
 
-        for num, subnode in enumerate(node.nodes):
+        for num, subnode in enumerate(node.children):
             name = subnode.name
             subval = value.pop(name, _missing)
 
@@ -168,7 +196,7 @@ class Mapping(object):
                     if subnode.required:
                         raise Invalid(
                             subnode,
-                            '%r is required but missing' % subnode.name)
+                            '"%s" is required but missing' % subnode.name)
                     result[name] = default_callback(subnode)
                 else:
                     result[name] = callback(subnode, subval)
@@ -228,7 +256,7 @@ class Tuple(Positional):
         if not hasattr(value, '__iter__'):
             raise Invalid(node, '%r is not iterable' % value)
 
-        valuelen, nodelen = len(value), len(node.nodes)
+        valuelen, nodelen = len(value), len(node.children)
 
         if valuelen != nodelen:
             raise Invalid(
@@ -243,7 +271,7 @@ class Tuple(Positional):
         error = None
         result = []
 
-        for num, subnode in enumerate(node.nodes):
+        for num, subnode in enumerate(node.children):
             subval = value[num]
             try:
                 result.append(callback(subnode, subval))
@@ -305,7 +333,7 @@ class Sequence(Positional):
         result = []
         for num, subval in enumerate(value):
             try:
-                result.append(callback(node.nodes[0], subval))
+                result.append(callback(node.children[0], subval))
             except Invalid, e:
                 if error is None:
                     error = Invalid(node)
@@ -378,13 +406,13 @@ class Integer(object):
                 if node.required:
                     raise Invalid(node, 'Required')
                 return node.default
-            raise Invalid(node, '%r is not a number' % value)
+            raise Invalid(node, '"%s" is not a number' % value)
 
     def serialize(self, node, value):
         try:
             return str(int(value))
         except Exception:
-            raise Invalid(node, '%r is not a number' % value)
+            raise Invalid(node, '"%s" is not a number' % value)
 
 Int = Integer
 
@@ -402,13 +430,13 @@ class Float(object):
                 if node.required:
                     raise Invalid(node, 'Required')
                 return node.default
-            raise Invalid(node, '%r is not a number' % value)
+            raise Invalid(node, '"%s" is not a number' % value)
 
     def serialize(self, node, value):
         try:
             return str(float(value))
         except Exception:
-            raise Invalid(node, '%r is not a number' % value)
+            raise Invalid(node, '"%s" is not a number' % value)
 
 Int = Integer
 
@@ -490,7 +518,7 @@ class GlobalObject(object):
             if not self.package:
                 raise Invalid(
                     node,
-                    'relative name %r irresolveable without package' % value)
+                    'relative name "%s" irresolveable without package' % value)
             if value in ['.', ':']:
                 value = self.package.__name__
             else:
@@ -505,7 +533,7 @@ class GlobalObject(object):
             if self.package is None:
                 raise Invalid(
                     node,
-                    "relative name %r irresolveable without package" % value)
+                    'relative name "%s" irresolveable without package' % value)
             name = module.split('.')
         else:
             name = value.split('.')
@@ -513,7 +541,7 @@ class GlobalObject(object):
                 if module is None:
                     raise Invalid(
                         node,
-                        "relative name %r irresolveable without package" %
+                        'relative name "%s" irresolveable without package' %
                         value)
                 module = module.split('.')
                 name.pop(0)
@@ -536,7 +564,7 @@ class GlobalObject(object):
 
     def deserialize(self, node, value):
         if not isinstance(value, basestring):
-            raise Invalid(node, '%r is not a string' % value)
+            raise Invalid(node, '"%s" is not a string' % value)
         try:
             if ':' in value:
                 return self._pkg_resources_style(node, value)
@@ -544,7 +572,7 @@ class GlobalObject(object):
                 return self._zope_dottedname_style(node, value)
         except ImportError:
             raise Invalid(node,
-                          'The dotted name %r cannot be imported' % value)
+                          'The dotted name "%s" cannot be imported' % value)
 
     def serialize(self, node, value):
         try:
@@ -562,9 +590,9 @@ class SchemaNode(object):
       instance of a class that implements the
       :class:`colander.interfaces.Type` interface.
 
-    - ``nodes``: a sequence of subnodes.  If the subnodes of this node
-      are not known at construction time, they can later be added via
-      the ``add`` method.
+    - ``children``: a sequence of subnodes.  If the subnodes of this
+      node are not known at construction time, they can later be added
+      via the ``add`` method.
 
     - ``name``: The name of this node.
 
@@ -575,6 +603,15 @@ class SchemaNode(object):
     - ``validator``: Optional validator for this node.  It should be
       an object that implements the
       :class:`colander.interfaces.Validator` interface.
+
+    - ``title``: The title of this node.  Defaults to a captialization
+      of the ``name``.  The title is used by higher-level systems (not
+      by Colander itself).
+
+    - ``description``: The description for this node.  Defaults to
+      ``''`` (the emtpty string).  The description is used by
+      higher-level systems (not by Colander itself).
+
     """
     
     _counter = itertools.count()
@@ -584,14 +621,14 @@ class SchemaNode(object):
         inst._order = cls._counter.next()
         return inst
         
-    def __init__(self, typ, *nodes, **kw):
+    def __init__(self, typ, *children, **kw):
         self.typ = typ
         self.validator = kw.get('validator', None)
         self.default = kw.get('default', _missing)
         self.name = kw.get('name', '')
         self.title = kw.get('title', self.name.capitalize())
         self.description = kw.get('description', '')
-        self.nodes = list(nodes)
+        self.children = list(children)
 
     def __repr__(self):
         return '<%s object at %x named %r>' % (self.__class__.__name__,
@@ -604,6 +641,14 @@ class SchemaNode(object):
         schema """
         return self.default is _missing
 
+    @property
+    def sdefault(self):
+        """ Return the *serialized* default of the node default or
+        ``None`` if there is no default."""
+        if self.default is _missing:
+            return None
+        return self.typ.serialize(self, self.default)
+
     def deserialize(self, value):
         """ Derialize the value based on the schema represented by this
         node """
@@ -619,14 +664,23 @@ class SchemaNode(object):
 
     def add(self, node):
         """ Add a subnode to this node """
-        self.nodes.append(node)
+        self.children.append(node)
 
     def __getitem__(self, name):
-        for node in self.nodes:
+        for node in self.children:
             if node.name == name:
                 return node
         raise KeyError(name)
 
+    def clone(self):
+        """ Clone the schema node and return the clone.  All subnodes
+        are also cloned recursively.  Attributes present in node
+        dictionaries are preserved."""
+        cloned = self.__class__(self.typ)
+        cloned.__dict__.update(self.__dict__)
+        cloned.children = [ node.clone() for node in self.children ]
+        return cloned
+
 class _SchemaMeta(type):
     def __init__(cls, name, bases, clsattrs):
         nodes = []
