@@ -81,7 +81,25 @@ class TestInvalid(unittest.TestCase):
         self.assertEqual(d, {'node1.node2.3': 'exc1; exc2; exc3',
                              'node1.node4': 'exc1; exc4'})
 
-
+    def test___str__(self):
+        from colander import Positional
+        node1 = DummySchemaNode(None, 'node1')
+        node2 = DummySchemaNode(Positional(), 'node2')
+        node3 = DummySchemaNode(Positional(), 'node3')
+        node4 = DummySchemaNode(Positional(), 'node4')
+        exc1 = self._makeOne(node1, 'exc1', pos=1)
+        exc2 = self._makeOne(node2, 'exc2', pos=2) 
+        exc3 = self._makeOne(node3, 'exc3', pos=3)
+        exc4 = self._makeOne(node4, 'exc4', pos=4)
+        exc1.add(exc2)
+        exc2.add(exc3)
+        exc1.add(exc4)
+        result = str(exc1)
+        self.assertEqual(
+            result,
+            "{'node1.node2.3': 'exc1; exc2; exc3', 'node1.node4': 'exc1; exc4'}"
+            )
+        
 class TestAll(unittest.TestCase):
     def _makeOne(self, validators):
         from colander import All
@@ -131,6 +149,37 @@ class TestRange(unittest.TestCase):
         e = invalid_exc(validator, None, 2)
         self.assertEqual(e.msg, '2 is greater than maximum value 1')
 
+class TestLength(unittest.TestCase):
+    def _makeOne(self, min=None, max=None):
+        from colander import Length
+        return Length(min=min, max=max)
+
+    def test_success_no_bounds(self):
+        validator = self._makeOne()
+        self.assertEqual(validator(None, ''), None)
+
+    def test_success_upper_bound_only(self):
+        validator = self._makeOne(max=1)
+        self.assertEqual(validator(None, 'a'), None)
+
+    def test_success_minimum_bound_only(self):
+        validator = self._makeOne(min=0)
+        self.assertEqual(validator(None, ''), None)
+
+    def test_success_min_and_max(self):
+        validator = self._makeOne(min=1, max=1)
+        self.assertEqual(validator(None, 'a'), None)
+
+    def test_min_failure(self):
+        validator = self._makeOne(min=1)
+        e = invalid_exc(validator, None, '')
+        self.assertEqual(e.msg, 'Shorter than minimum length 1')
+
+    def test_max_failure(self):
+        validator = self._makeOne(max=1)
+        e = invalid_exc(validator, None, 'ab')
+        self.assertEqual(e.msg, 'Longer than maximum length 1')
+
 class TestOneOf(unittest.TestCase):
     def _makeOne(self, values):
         from colander import OneOf
@@ -176,28 +225,28 @@ class TestMapping(unittest.TestCase):
 
     def test_deserialize_ok(self):
         node = DummySchemaNode(None)
-        node.nodes = [DummySchemaNode(None, name='a')]
+        node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne()
         result = typ.deserialize(node, {'a':1})
         self.assertEqual(result, {'a':1})
 
     def test_deserialize_unknown_keys_raise(self):
         node = DummySchemaNode(None)
-        node.nodes = [DummySchemaNode(None, name='a')]
+        node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne('raise')
         e = invalid_exc(typ.deserialize, node, {'a':1, 'b':2})
         self.assertEqual(e.msg, "Unrecognized keys in mapping: {'b': 2}")
 
     def test_deserialize_unknown_keys_preserve(self):
         node = DummySchemaNode(None)
-        node.nodes = [DummySchemaNode(None, name='a')]
+        node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne('preserve')
         result = typ.deserialize(node, {'a':1, 'b':2})
         self.assertEqual(result, {'a':1, 'b':2})
 
     def test_deserialize_subnodes_raise(self):
         node = DummySchemaNode(None)
-        node.nodes = [
+        node.children = [
             DummySchemaNode(None, name='a', exc='Wrong 2'),
             DummySchemaNode(None, name='b', exc='Wrong 2'),
             ]
@@ -208,7 +257,7 @@ class TestMapping(unittest.TestCase):
 
     def test_deserialize_subnode_missing_default(self):
         node = DummySchemaNode(None)
-        node.nodes = [
+        node.children = [
             DummySchemaNode(None, name='a'),
             DummySchemaNode(None, name='b', default='abc'),
             ]
@@ -218,13 +267,13 @@ class TestMapping(unittest.TestCase):
 
     def test_deserialize_subnode_missing_nodefault(self):
         node = DummySchemaNode(None)
-        node.nodes = [
+        node.children = [
             DummySchemaNode(None, name='a'),
             DummySchemaNode(None, name='b'),
             ]
         typ = self._makeOne()
         e = invalid_exc(typ.deserialize, node, {'a':1})
-        self.assertEqual(e.children[0].msg, "'b' is required but missing")
+        self.assertEqual(e.children[0].msg, '"b" is required but missing')
 
     def test_serialize_not_a_mapping(self):
         node = DummySchemaNode(None)
@@ -241,28 +290,28 @@ class TestMapping(unittest.TestCase):
 
     def test_serialize_ok(self):
         node = DummySchemaNode(None)
-        node.nodes = [DummySchemaNode(None, name='a')]
+        node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne()
         result = typ.serialize(node, {'a':1})
         self.assertEqual(result, {'a':1})
 
     def test_serialize_unknown_keys_raise(self):
         node = DummySchemaNode(None)
-        node.nodes = [DummySchemaNode(None, name='a')]
+        node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne('raise')
         e = invalid_exc(typ.serialize, node, {'a':1, 'b':2})
         self.assertEqual(e.msg, "Unrecognized keys in mapping: {'b': 2}")
 
     def test_serialize_unknown_keys_preserve(self):
         node = DummySchemaNode(None)
-        node.nodes = [DummySchemaNode(None, name='a')]
+        node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne('preserve')
         result = typ.serialize(node, {'a':1, 'b':2})
         self.assertEqual(result, {'a':1, 'b':2})
 
     def test_serialize_subnodes_raise(self):
         node = DummySchemaNode(None)
-        node.nodes = [
+        node.children = [
             DummySchemaNode(None, name='a', exc='Wrong 2'),
             DummySchemaNode(None, name='b', exc='Wrong 2'),
             ]
@@ -273,7 +322,7 @@ class TestMapping(unittest.TestCase):
 
     def test_serialize_subnode_missing_default(self):
         node = DummySchemaNode(None)
-        node.nodes = [
+        node.children = [
             DummySchemaNode(None, name='a'),
             DummySchemaNode(None, name='b', default='abc'),
             ]
@@ -283,13 +332,13 @@ class TestMapping(unittest.TestCase):
 
     def test_serialize_subnode_missing_nodefault(self):
         node = DummySchemaNode(None)
-        node.nodes = [
+        node.children = [
             DummySchemaNode(None, name='a'),
             DummySchemaNode(None, name='b'),
             ]
         typ = self._makeOne()
         e = invalid_exc(typ.serialize, node, {'a':1})
-        self.assertEqual(e.children[0].msg, "'b' is required but missing")
+        self.assertEqual(e.children[0].msg, '"b" is required but missing')
 
 class TestTuple(unittest.TestCase):
     def _makeOne(self):
@@ -313,14 +362,14 @@ class TestTuple(unittest.TestCase):
 
     def test_deserialize_ok(self):
         node = DummySchemaNode(None)
-        node.nodes = [DummySchemaNode(None, name='a')]
+        node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne()
         result = typ.deserialize(node, ('a',))
         self.assertEqual(result, ('a',))
 
     def test_deserialize_toobig(self):
         node = DummySchemaNode(None)
-        node.nodes = [DummySchemaNode(None, name='a')]
+        node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne()
         e = invalid_exc(typ.deserialize, node, ('a','b'))
         self.assertEqual(e.msg,
@@ -328,7 +377,7 @@ class TestTuple(unittest.TestCase):
 
     def test_deserialize_toosmall(self):
         node = DummySchemaNode(None)
-        node.nodes = [DummySchemaNode(None, name='a')]
+        node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne()
         e = invalid_exc(typ.deserialize, node, ())
         self.assertEqual(e.msg,
@@ -336,7 +385,7 @@ class TestTuple(unittest.TestCase):
 
     def test_deserialize_subnodes_raise(self):
         node = DummySchemaNode(None)
-        node.nodes = [
+        node.children = [
             DummySchemaNode(None, name='a', exc='Wrong 2'),
             DummySchemaNode(None, name='b', exc='Wrong 2'),
             ]
@@ -362,14 +411,14 @@ class TestTuple(unittest.TestCase):
 
     def test_serialize_ok(self):
         node = DummySchemaNode(None)
-        node.nodes = [DummySchemaNode(None, name='a')]
+        node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne()
         result = typ.serialize(node, ('a',))
         self.assertEqual(result, ('a',))
 
     def test_serialize_toobig(self):
         node = DummySchemaNode(None)
-        node.nodes = [DummySchemaNode(None, name='a')]
+        node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne()
         e = invalid_exc(typ.serialize, node, ('a','b'))
         self.assertEqual(e.msg,
@@ -377,7 +426,7 @@ class TestTuple(unittest.TestCase):
 
     def test_serialize_toosmall(self):
         node = DummySchemaNode(None)
-        node.nodes = [DummySchemaNode(None, name='a')]
+        node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne()
         e = invalid_exc(typ.serialize, node, ())
         self.assertEqual(e.msg,
@@ -385,7 +434,7 @@ class TestTuple(unittest.TestCase):
 
     def test_serialize_subnodes_raise(self):
         node = DummySchemaNode(None)
-        node.nodes = [
+        node.children = [
             DummySchemaNode(None, name='a', exc='Wrong 2'),
             DummySchemaNode(None, name='b', exc='Wrong 2'),
             ]
@@ -407,7 +456,7 @@ class TestSequence(unittest.TestCase):
     def test_deserialize_not_iterable(self):
         node = DummySchemaNode(None)
         typ = self._makeOne()
-        node.nodes = [node]
+        node.children = [node]
         e = invalid_exc(typ.deserialize, node, None)
         self.assertEqual(
             e.msg,
@@ -417,29 +466,29 @@ class TestSequence(unittest.TestCase):
     def test_deserialize_not_iterable_accept_scalar(self):
         node = DummySchemaNode(None)
         typ = self._makeOne(accept_scalar=True)
-        node.nodes = [node]
+        node.children = [node]
         result = typ.deserialize(node, None)
         self.assertEqual(result, [None])
 
     def test_deserialize_no_subnodes(self):
         typ = self._makeOne()
         node = DummySchemaNode(None)
-        node.nodes = [node]
+        node.children = [node]
         result = typ.deserialize(node, ())
         self.assertEqual(result, [])
 
     def test_deserialize_ok(self):
         node = DummySchemaNode(None)
-        node.nodes = [DummySchemaNode(None, name='a')]
+        node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne()
-        node.nodes = [node]
+        node.children = [node]
         result = typ.deserialize(node, ('a',))
         self.assertEqual(result, ['a'])
 
     def test_deserialize_subnodes_raise(self):
         node = DummySchemaNode(None, exc='Wrong')
         typ = self._makeOne()
-        node.nodes = [node]
+        node.children = [node]
         e = invalid_exc(typ.deserialize, node, ('1', '2'))
         self.assertEqual(e.msg, None)
         self.assertEqual(len(e.children), 2)
@@ -447,7 +496,7 @@ class TestSequence(unittest.TestCase):
     def test_serialize_not_iterable(self):
         node = DummySchemaNode(None)
         typ = self._makeOne()
-        node.nodes = [node]
+        node.children = [node]
         e = invalid_exc(typ.serialize, node, None)
         self.assertEqual(
             e.msg,
@@ -457,20 +506,20 @@ class TestSequence(unittest.TestCase):
     def test_serialize_not_iterable_accept_scalar(self):
         node = DummySchemaNode(None)
         typ = self._makeOne(accept_scalar=True)
-        node.nodes = [node]
+        node.children = [node]
         result = typ.serialize(node, None)
         self.assertEqual(result, [None])
 
     def test_serialize_no_subnodes(self):
         node = DummySchemaNode(None)
-        node.nodes = [node]
+        node.children = [node]
         typ = self._makeOne()
         result = typ.serialize(node, ())
         self.assertEqual(result, [])
 
     def test_serialize_ok(self):
         node = DummySchemaNode(None)
-        node.nodes = [DummySchemaNode(None, name='a')]
+        node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne()
         result = typ.serialize(node, ('a',))
         self.assertEqual(result, ['a'])
@@ -478,7 +527,7 @@ class TestSequence(unittest.TestCase):
     def test_serialize_subnodes_raise(self):
         node = DummySchemaNode(None, exc='Wrong')
         typ = self._makeOne()
-        node.nodes = [node]
+        node.children = [node]
         e = invalid_exc(typ.serialize, node, ('1', '2'))
         self.assertEqual(e.msg, None)
         self.assertEqual(len(e.children), 2)
@@ -745,13 +794,13 @@ class TestGlobalObject(unittest.TestCase):
         e = invalid_exc(typ._zope_dottedname_style, None, '.')
         self.assertEqual(
             e.msg,
-            "relative name '.' irresolveable without package")
+            'relative name "." irresolveable without package')
 
     def test_zope_dottedname_style_resolve_relative_nocurrentpackage(self):
         typ = self._makeOne()
         e = invalid_exc(typ._zope_dottedname_style, None, '.whatever')
         self.assertEqual(
-            e.msg, "relative name '.whatever' irresolveable without package")
+            e.msg, 'relative name ".whatever" irresolveable without package')
 
     def test_zope_dottedname_style_irrresolveable_relative(self):
         import colander.tests
@@ -821,7 +870,7 @@ class TestGlobalObject(unittest.TestCase):
     def test_deserialize_not_a_string(self):
         typ = self._makeOne()
         e = invalid_exc(typ.deserialize, None, None)
-        self.assertEqual(e.msg, "None is not a string")
+        self.assertEqual(e.msg, '"None" is not a string')
 
     def test_deserialize_using_pkgresources_style(self):
         typ = self._makeOne()
@@ -837,7 +886,7 @@ class TestGlobalObject(unittest.TestCase):
         typ = self._makeOne()
         e = invalid_exc(typ.deserialize, None, 'cant.be.found')
         self.assertEqual(e.msg,
-                         "The dotted name 'cant.be.found' cannot be imported")
+                         'The dotted name "cant.be.found" cannot be imported')
 
     def test_serialize_ok(self):
         import colander.tests
@@ -862,7 +911,7 @@ class TestSchemaNode(unittest.TestCase):
     def test_ctor_no_title(self):
         node = self._makeOne(None, 0, validator=1, default=2, name='name')
         self.assertEqual(node.typ, None)
-        self.assertEqual(node.nodes, [0])
+        self.assertEqual(node.children, [0])
         self.assertEqual(node.validator, 1)
         self.assertEqual(node.default, 2)
         self.assertEqual(node.name, 'name')
@@ -872,7 +921,7 @@ class TestSchemaNode(unittest.TestCase):
         node = self._makeOne(None, 0, validator=1, default=2, name='name',
                              title='title')
         self.assertEqual(node.typ, None)
-        self.assertEqual(node.nodes, [0])
+        self.assertEqual(node.children, [0])
         self.assertEqual(node.validator, 1)
         self.assertEqual(node.default, 2)
         self.assertEqual(node.name, 'name')
@@ -891,6 +940,15 @@ class TestSchemaNode(unittest.TestCase):
         node = self._makeOne(None, default=1)
         self.assertEqual(node.required, False)
 
+    def test_sdefault_missing(self):
+        node = self._makeOne(None)
+        self.assertEqual(node.sdefault, None)
+
+    def test_sdefault_not_missing(self):
+        typ = DummyType()
+        node = self._makeOne(typ, default=1)
+        self.assertEqual(node.sdefault, 1)
+
     def test_deserialize_no_validator(self):
         typ = DummyType()
         node = self._makeOne(typ)
@@ -913,7 +971,7 @@ class TestSchemaNode(unittest.TestCase):
     def test_add(self):
         node = self._makeOne(None)
         node.add(1)
-        self.assertEqual(node.nodes, [1])
+        self.assertEqual(node.children, [1])
 
     def test_repr(self):
         node = self._makeOne(None, name='flub')
@@ -931,6 +989,26 @@ class TestSchemaNode(unittest.TestCase):
         node = self._makeOne(None)
         self.assertRaises(KeyError, node.__getitem__, 'another')
 
+    def test_clone(self):
+        inner_typ = DummyType()
+        outer_typ = DummyType()
+        outer_node = self._makeOne(outer_typ, name='outer')
+        inner_node = self._makeOne(inner_typ, name='inner')
+        outer_node.foo = 1
+        inner_node.foo = 2
+        outer_node.children = [inner_node]
+        outer_clone = outer_node.clone()
+        self.failIf(outer_clone is outer_node)
+        self.assertEqual(outer_clone.typ, outer_typ)
+        self.assertEqual(outer_clone.name, 'outer')
+        self.assertEqual(outer_node.foo, 1)
+        self.assertEqual(len(outer_clone.children), 1)
+        inner_clone = outer_clone.children[0]
+        self.failIf(inner_clone is inner_node)
+        self.assertEqual(inner_clone.typ, inner_typ)
+        self.assertEqual(inner_clone.name, 'inner')
+        self.assertEqual(inner_clone.foo, 2)
+
 class TestSchema(unittest.TestCase):
     def test_alias(self):
         from colander import Schema
@@ -947,9 +1025,9 @@ class TestSchema(unittest.TestCase):
         self.assertEqual(node.__class__, colander.SchemaNode)
         self.assertEqual(node.typ.__class__, colander.Mapping)
         self.assertEqual(node.typ.unknown_keys, 'raise')
-        self.assertEqual(node.nodes[0].typ.__class__, colander.String) 
-        self.assertEqual(node.nodes[0].title, 'Thing')
-        self.assertEqual(node.nodes[1].title, 'bar')
+        self.assertEqual(node.children[0].typ.__class__, colander.String) 
+        self.assertEqual(node.children[0].title, 'Thing')
+        self.assertEqual(node.children[1].title, 'bar')
         
 class TestSequenceSchema(unittest.TestCase):
     def test_it(self):
@@ -961,7 +1039,7 @@ class TestSequenceSchema(unittest.TestCase):
         self.failUnless(hasattr(node, '_order'))
         self.assertEqual(node.__class__, colander.SchemaNode)
         self.assertEqual(node.typ.__class__, colander.Sequence)
-        self.assertEqual(node.nodes[0], _inner)
+        self.assertEqual(node.children[0], _inner)
 
 class TestTupleSchema(unittest.TestCase):
     def test_it(self):
@@ -972,7 +1050,7 @@ class TestTupleSchema(unittest.TestCase):
         self.failUnless(hasattr(node, '_order'))
         self.assertEqual(node.__class__, colander.SchemaNode)
         self.assertEqual(node.typ.__class__, colander.Tuple)
-        self.assertEqual(node.nodes[0].typ.__class__, colander.String)
+        self.assertEqual(node.children[0].typ.__class__, colander.String)
 
 class TestFunctional(object):
     def test_deserialize_ok(self):
@@ -997,16 +1075,16 @@ class TestFunctional(object):
     def test_invalid_asdict(self):
         expected = {
             'int': '20 is greater than maximum value 10',
-            'ob': "The dotted name 'no.way.this.exists' cannot be imported",
-            'seq.0.0': "'q' is not a number",
-            'seq.1.0': "'w' is not a number",
-            'seq.2.0': "'e' is not a number",
-            'seq.3.0': "'r' is not a number",
-            'seq2.0.key': "'t' is not a number",
-            'seq2.0.key2': "'y' is not a number",
-            'seq2.1.key': "'u' is not a number",
-            'seq2.1.key2': "'i' is not a number",
-            'tup.0': "'s' is not a number"}
+            'ob': 'The dotted name "no.way.this.exists" cannot be imported',
+            'seq.0.0': '"q" is not a number',
+            'seq.1.0': '"w" is not a number',
+            'seq.2.0': '"e" is not a number',
+            'seq.3.0': '"r" is not a number',
+            'seq2.0.key': '"t" is not a number',
+            'seq2.0.key2': '"y" is not a number',
+            'seq2.1.key': '"u" is not a number',
+            'seq2.1.key2': '"i" is not a number',
+            'tup.0': '"s" is not a number'}
         data = {
             'int':'20',
             'ob':'no.way.this.exists',
@@ -1122,7 +1200,7 @@ class DummySchemaNode(object):
         self.exc = exc
         self.required = default is None
         self.default = default
-        self.nodes = []
+        self.children = []
 
     def deserialize(self, val):
         from colander import Invalid
